@@ -1,22 +1,56 @@
-//! Perchance is a simple random number generation library, tuned for ease of use.
-//! Create an instance and go.
+//! `perchance` is a simple random number generation library, tuned for ease of use: create an instance of [`PerchanceContext`] and go.
 //!
-//! If built for `wasm32` arch, there's a global context, too. WASM modules are single threaded
-//! so using the global context will actually be safe.
-//! On other architectures, simply make an instance of [`PerchanceContext`] and manage it
-//! yourself.
+//! Note that `perchance` is **not** cryptographically secure and ***should not*** be used in any context where security is a concern.
+//!
+//! When the `std` feature is enabled (by default), there's a global [`PerchanceContext`] provided, too.
+//!
+//! # Example
+//!
+//! First, create a [`PerchanceContext`]:
+//!
+//! ```rust
+//! # let my_seed = 0;
+//! let mut rng = perchance::PerchanceContext::new(my_seed);
+//! ```
+//!
+//! Or, if you have the `std` feature enabled, obtain the global [`PerchanceContext`] object:
+//!
+//! ```rust
+//! // Seed the global context first. You may do so manually, or, on platforms that support it,
+//! // obtain a seed to pass into it by calling `perchance::gen_time_seed()`.
+//! perchance::seed_global(0x5F3759DF); // ;)
+//!
+//! let mut rng = perchance::global();
+//! ```
+//!
+//! Then, start letting things happen perchance!
+//!
+//! ```rust
+//! # perchance::seed_global(0x5F3759DF);
+//! # let mut rng = perchance::global();
+//! let between_0_and_1 = rng.uniform_f32();
+//! let dice_roll = rng.uniform_range_i32(1..=6);
+//! let random_direction = rng.uniform_sphere_surface_vec3();
+//! let thing_should_happen = rng.get_bool();
+//!
+//! enum Event {
+//!     Thing1,
+//!     Thing2,
+//!     Thing3,
+//! }
+//! let which_should_happen = rng.choose(&[Event::Thing1, Event::Thing2, Event::Thing3]);
+//! ```
 //!
 //! Some of the convenience functions exist in multiple flavors with different
 //! return types.
 //!
 //! Perchance does NOT guarantee the same random number generator across new versions of the crate.
-
+#![cfg_attr(not(feature = "std"), no_std)]
 #![warn(rust_2018_idioms, clippy::all, clippy::needless_pass_by_value)]
-#![allow(clippy::disallowed_type)] // allow usage of std::sync::Mutex (instead of parking_lot) as this is typically intended for WASM
+#![allow(clippy::disallowed_types)] // allow usage of std::sync::Mutex (instead of parking_lot) as this is typically intended for WASM
 
+use core::ops::{Bound, RangeBounds};
 use macaw::{BoundingBox, Vec2, Vec3};
-use ordered_float::NotNan;
-use std::ops::Range;
 
 /// The raw random number generator.
 /// As of may 2020, this is the same generator used by `rand::SmallRng`.
@@ -54,6 +88,49 @@ impl Rng {
     }
 }
 
+macro_rules! impl_get_bounds {
+    (float, $fn_name:ident, $t:ty) => {
+        /// Returned start is inclusive, returned end is exclusive
+        /// Ignores inclusivity of input bounds.
+        /// Unbounded gets set to `$t::MIN` or `$t::MAX` respectively.
+        fn $fn_name<R: RangeBounds<$t>>(bounds: R) -> ($t, $t) {
+            let start = match bounds.start_bound() {
+                Bound::Included(&b) | Bound::Excluded(&b) => b,
+                Bound::Unbounded => <$t>::MIN,
+            };
+            let end = match bounds.end_bound() {
+                Bound::Included(&b) | Bound::Excluded(&b) => b,
+                Bound::Unbounded => <$t>::MAX,
+            };
+            (start, end)
+        }
+    };
+    (int, $fn_name:ident, $t:ty) => {
+        /// Returned start is inclusive, returned end is exclusive.
+        /// Unbounded gets set to `$t::MIN` or `$t::MAX` respectively.
+        /// Panics on overflow, if:
+        /// - lower bound exclusive and = `$t::MAX`
+        /// - upper bound inclusive and = `$t::MAX`
+        fn $fn_name<R: RangeBounds<$t>>(bounds: R) -> ($t, $t) {
+            let start = match bounds.start_bound() {
+                Bound::Included(&b) => b,
+                Bound::Excluded(&b) => b.checked_add(1).unwrap(),
+                Bound::Unbounded => <$t>::MIN,
+            };
+            let end = match bounds.start_bound() {
+                Bound::Included(&b) => b.checked_add(1).unwrap(),
+                Bound::Excluded(&b) => b,
+                Bound::Unbounded => <$t>::MAX,
+            };
+            (start, end)
+        }
+    };
+}
+
+impl_get_bounds!(float, get_f32_bounds, f32);
+impl_get_bounds!(int, get_usize_bounds, usize);
+impl_get_bounds!(int, get_i32_bounds, i32);
+
 #[derive(Clone)]
 pub struct PerchanceContext(Rng);
 
@@ -149,12 +226,13 @@ impl PerchanceContext {
     }
 
     /// Returns a random integer uniformly distributed over `range`.
-    /// Panics if the range is invalid (end < start).
+    /// Panics if the range is invalid (end < start) or if the range bounds would overflow an i32.
     /// Example: `let dice = rng.uniform_range_i32(0..6);`
-    pub fn uniform_range_i32(&mut self, range: Range<i32>) -> i32 {
+    pub fn uniform_range_i32<R: RangeBounds<i32>>(&mut self, range: R) -> i32 {
         // Switch to i64 to prevent overflows
-        let start = range.start as i64;
-        let end = range.end as i64;
+        let (start, end) = get_i32_bounds(range);
+        let start = start as i64;
+        let end = end as i64;
         let range_size = end - start;
         assert!(range_size >= 0);
         let value = start + self.u64_less_than(range_size as u64) as i64;
@@ -162,11 +240,12 @@ impl PerchanceContext {
     }
 
     /// Returns a random usize uniformly distributed over `range`.
-    /// Panics if the range is invalid (end < start).
+    /// Panics if the range is invalid (end < start) or if the range bounds would overflow a usize.
     /// Example: `let array_index = rng.uniform_range_usize(0..array.len());`
-    pub fn uniform_range_usize(&mut self, range: Range<usize>) -> usize {
-        assert!(range.end >= range.start);
-        range.start + self.usize_less_than(range.len())
+    pub fn uniform_range_usize<R: RangeBounds<usize>>(&mut self, range: R) -> usize {
+        let (start, end) = get_usize_bounds(range);
+        assert!(end >= start);
+        start + self.usize_less_than(end - start)
     }
 
     /// Returns a random floating point number in the range [0.0, 1.0).
@@ -186,8 +265,17 @@ impl PerchanceContext {
     }
 
     /// Returns a random floating point number uniformly distributed over `range`.
-    pub fn uniform_range_f32(&mut self, range: Range<f32>) -> f32 {
-        self.uniform_f32() * (range.end - range.start) + range.start
+    ///
+    /// Note that this ignores the inclusivity of the range (always assumes an inclusive
+    /// lower bound and an exclusive upper bound). An unbonded lower and upper bound will be set to
+    /// `f32::MIN` and `f32::MAX` respectively.
+    pub fn uniform_range_f32<R: RangeBounds<f32>>(&mut self, range: R) -> f32 {
+        self.uniform_range_f32_raw(get_f32_bounds(range))
+    }
+
+    #[inline]
+    fn uniform_range_f32_raw(&mut self, (start, end): (f32, f32)) -> f32 {
+        self.uniform_f32() * (start - end) + start
     }
 
     /// A normally distributed value with standard deviation=1
@@ -204,7 +292,7 @@ impl PerchanceContext {
             r = (-2.0 * self.uniform_f64().ln()).sqrt();
             !r.is_finite()
         } {}
-        let angle = 2.0 * std::f64::consts::PI * self.uniform_f64();
+        let angle = 2.0 * core::f64::consts::PI * self.uniform_f64();
         let (s, c) = angle.sin_cos();
         Vec2::new((r * s) as f32, (r * c) as f32)
     }
@@ -215,36 +303,38 @@ impl PerchanceContext {
     }
 
     /// Returns a Vec2 where both components are uniformly distributed over `range`.
-    pub fn uniform_square_vec2(&mut self, range: Range<f32>) -> Vec2 {
+    pub fn uniform_square_vec2<R: RangeBounds<f32>>(&mut self, range: R) -> Vec2 {
+        let bounds = get_f32_bounds(range);
         Vec2::new(
-            self.uniform_range_f32(range.clone()),
-            self.uniform_range_f32(range),
+            self.uniform_range_f32_raw(bounds),
+            self.uniform_range_f32_raw(bounds),
         )
     }
 
     /// Returns a Vec3 where all three components are uniformly distributed over `range`.
-    pub fn uniform_cube_vec3(&mut self, range: Range<f32>) -> Vec3 {
+    pub fn uniform_cube_vec3<R: RangeBounds<f32>>(&mut self, range: R) -> Vec3 {
+        let bounds = get_f32_bounds(range);
         Vec3::new(
-            self.uniform_range_f32(range.clone()),
-            self.uniform_range_f32(range.clone()),
-            self.uniform_range_f32(range),
+            self.uniform_range_f32_raw(bounds),
+            self.uniform_range_f32_raw(bounds),
+            self.uniform_range_f32_raw(bounds),
         )
     }
 
     /// Returns a uniformly random point that lies inside the bounding box.
     pub fn uniform_bounds_vec3(&mut self, bounds: BoundingBox) -> Vec3 {
         Vec3::new(
-            self.uniform_range_f32(bounds.min.x..bounds.max.x),
-            self.uniform_range_f32(bounds.min.y..bounds.max.y),
-            self.uniform_range_f32(bounds.min.z..bounds.max.z),
+            self.uniform_range_f32_raw((bounds.min.x, bounds.max.x)),
+            self.uniform_range_f32_raw((bounds.min.y, bounds.max.y)),
+            self.uniform_range_f32_raw((bounds.min.z, bounds.max.z)),
         )
     }
 
     /// Returns a uniformly random point that lies inside the rectangle.
     pub fn uniform_rectangle_vec2(&mut self, min: Vec2, max: Vec2) -> Vec2 {
         Vec2::new(
-            self.uniform_range_f32(min.x..max.x),
-            self.uniform_range_f32(min.y..max.y),
+            self.uniform_range_f32_raw((min.x, max.x)),
+            self.uniform_range_f32_raw((min.y, max.y)),
         )
     }
 
@@ -263,7 +353,7 @@ impl PerchanceContext {
     pub fn uniform_sphere_surface_vec3(&mut self) -> Vec3 {
         let x = self.uniform_f32();
         let y = self.uniform_f32();
-        let tau = 2.0 * std::f32::consts::PI;
+        let tau = 2.0 * core::f32::consts::PI;
         let z = 1.0f32 - 2.0f32 * x;
         let xy = ((0.0f32).max(1.0f32 - z * z)).sqrt();
         let sn = (tau * y).sin();
@@ -316,7 +406,9 @@ impl PerchanceContext {
     /// Comlexity for `N` samples with `C` weights is `C + N*log(C)` instead of `C + N*C` when using `weighted_choice_f32`.
     ///
     /// Panics if given a NaN weight.
+    #[cfg(feature = "std")]
     pub fn weighted_choices_f32(&mut self, weights: &[f32]) -> WeightedSampler<'_> {
+        use ordered_float::NotNan;
         assert!(!weights.is_empty());
         let zero = NotNan::<f64>::default();
         let mut sum = zero;
@@ -334,11 +426,13 @@ impl PerchanceContext {
     }
 }
 
+#[cfg(feature = "std")]
 pub struct WeightedSampler<'a> {
     context: &'a mut PerchanceContext,
-    cumulative_weights: Vec<NotNan<f64>>,
+    cumulative_weights: Vec<ordered_float::NotNan<f64>>,
 }
 
+#[cfg(feature = "std")]
 impl<'a> WeightedSampler<'a> {
     /// Returns an index sampled according to the given weights.
     ///
@@ -346,43 +440,68 @@ impl<'a> WeightedSampler<'a> {
     pub fn sample(&mut self) -> usize {
         // Note: guaranteed to contain at least one element
         let weight_sum = *self.cumulative_weights.last().unwrap();
-        let value = NotNan::new(self.context.uniform_f64()).unwrap() * weight_sum;
+        let value = ordered_float::NotNan::new(self.context.uniform_f64()).unwrap() * weight_sum;
         match self.cumulative_weights.binary_search(&value) {
             Ok(index) | Err(index) => index,
         }
     }
 }
 
-use once_cell::sync::Lazy;
-use std::sync::Mutex;
-static GLOBAL_RNG: Lazy<Mutex<PerchanceContext>> =
-    Lazy::new(|| Mutex::new(PerchanceContext::new(0)));
-static HAS_BEEN_SEEDED: std::sync::atomic::AtomicBool = std::sync::atomic::AtomicBool::new(false);
+#[cfg(feature = "std")]
+mod global_ctx {
+    use super::*;
+    use once_cell::sync::Lazy;
+    use std::sync::Mutex;
+    use std::time::{SystemTime, UNIX_EPOCH};
 
-/// Returns the global `PerchanceContext`.
-///
-/// Call methods on it like [`PerchanceContext::get_u32`] to get random numbers from anywhere in your module.
-///
-/// You must call `perchance::seed(my_seed);` before using this, or this call will panic.
-pub fn global() -> std::sync::MutexGuard<'static, PerchanceContext> {
-    assert!(
-        has_been_seeded(),
-        "You must call `perchance::seed` before using the global perchance instance."
-    );
-    GLOBAL_RNG.lock().unwrap()
+    const TWO_POW_64: u128 = 2u128.pow(64);
+    const TWO_POW_96: u128 = 2u128.pow(96);
+
+    static GLOBAL_RNG: Lazy<Mutex<PerchanceContext>> =
+        Lazy::new(|| Mutex::new(PerchanceContext::new(0)));
+    static HAS_BEEN_SEEDED: std::sync::atomic::AtomicBool =
+        std::sync::atomic::AtomicBool::new(false);
+
+    /// Returns the global [`PerchanceContext`]. You **must** seed it by calling
+    /// [`perchance::seed_global`](self::seed_global) at least once before calling this function or else
+    /// it will panic.
+    ///
+    /// Call methods on it like [`PerchanceContext::get_u32`] to get random numbers from anywhere in your module.
+    pub fn global<'a>() -> std::sync::MutexGuard<'a, PerchanceContext> {
+        assert!(
+            global_has_been_seeded(),
+            "You must call `perchance::seed_global` before using the global perchance instance."
+        );
+        GLOBAL_RNG.lock().unwrap()
+    }
+
+    /// Seed (or reseed) the global [`PerchanceContext`] returned by [`perchance::global`](self::global).
+    ///
+    /// After calling this at least once, [`perchance::global_has_been_seeded`](self::global_has_been_seeded) will return true.
+    pub fn seed_global(seed: u128) {
+        HAS_BEEN_SEEDED.store(true, std::sync::atomic::Ordering::SeqCst);
+        *global() = PerchanceContext::new(seed);
+    }
+
+    /// Generates a seed that may be passed to [`perchance::seed_global`](self::seed_global) based on the
+    /// system clock.
+    #[cfg(not(wasm))]
+    pub fn gen_time_seed() -> u128 {
+        SystemTime::now()
+            .duration_since(UNIX_EPOCH)
+            .map(|duration| {
+                duration.as_secs() as u128 // u64 as bits 0..64 of u128
+                    | ((duration.subsec_millis() as u128) * TWO_POW_64) // put u32 into bits 64..96 of u128
+                    | ((duration.subsec_micros() as u128) * TWO_POW_96) // put u32 into bits 96..128 of u128
+            })
+            .unwrap()
+    }
+
+    /// Returns `true` if [`perchance::seed_global`](self::seed_global) has been called at least once.
+    pub fn global_has_been_seeded() -> bool {
+        HAS_BEEN_SEEDED.load(std::sync::atomic::Ordering::SeqCst)
+    }
 }
 
-/// Seed (or reseed) the global `PerchanceContext` returned by `perchance::global()`.
-///
-/// You must call this at least once before calling `perchance::global()`.
-///
-/// After this call `perchance::has_been_seeded()` will return true.
-pub fn seed(seed: u128) {
-    HAS_BEEN_SEEDED.store(true, std::sync::atomic::Ordering::SeqCst);
-    *global() = PerchanceContext::new(seed);
-}
-
-/// Returns `true` if `perchance::seed()` has been called.
-pub fn has_been_seeded() -> bool {
-    HAS_BEEN_SEEDED.load(std::sync::atomic::Ordering::SeqCst)
-}
+#[cfg(feature = "std")]
+pub use global_ctx::*;
